//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by test3.rc
//
#define IDM_ABOUTBOX                    0x0010
#define IDD_ABOUTBOX                    100
#define IDS_ABOUTBOX                    101
#define IDD_TEST3_DIALOG                102
#define IDR_MAINFRAME                   128
#define IDC_BUTTON1                     1000
#define IDC_EDIT1                       1001
#define IDC_BUTTON2                     1003
#define IDC_RICHEDIT21                  1004
#define IDC_EDIT2                       1006
#define IDC_BUTTON3                     1007
#define IDC_RICHEDIT22                  1009
#define IDC_EDIT4                       1011
#define IDC_EDIT3                       1013

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        129
#define _APS_NEXT_COMMAND_VALUE         32771
#define _APS_NEXT_CONTROL_VALUE         1014
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
