// MyEdit.cpp : implementation file
//

#include "stdafx.h"
#include "test3.h"
#include "MyEdit.h"


// MyEdit

IMPLEMENT_DYNAMIC(MyEdit, CRichEditCtrl)
MyEdit::MyEdit()
{
}

MyEdit::~MyEdit()
{
}


BEGIN_MESSAGE_MAP(MyEdit, CRichEditCtrl)
END_MESSAGE_MAP()



// MyEdit message handlers

